// src/index.ts
import {
  delimitedIndent,
  foldInside,
  foldNodeProp,
  indentNodeProp,
  LanguageSupport,
  LRLanguage
} from "@codemirror/language";
import { styleTags, tags as t } from "@lezer/highlight";

// esbuild-plugin-lezer:/Users/nvie/Projects/liveblocks/liveblocks-schema/packages/codemirror-language/src/syntax.grammar
import { LRParser } from "@lezer/lr";
var spec_name = { __proto__: null, type: 10 };
var parser = LRParser.deserialize({
  version: 14,
  states: "'UO]QPOOObQQO'#C_Q]QPOOOOQO'#C`'#C`OOQO'#Cp'#CpOOQO'#Cb'#CbOgQPO,58yOOQO-E6n-E6nOoQPO'#C|OOQO'#Cc'#CcOOQO1G.e1G.eOtQPO1G.eOOQO'#Cf'#CfOyQPO'#CeOOQO'#Cq'#CqO!RQPO,59hOOQO7+$P7+$PO!ZQPO,59PO!cQPO,59POOQO-E6o-E6oOOQO1G/S1G/SOOQO'#Ci'#CiO!hQPO'#ChOOQO'#Ch'#ChO!|QPO'#DOO#[QPO1G.kO!ZQPO1G.kO!ZQPO,59VOOQO'#Cs'#CsO#gQPO,59XO!ZQPO'#CtO#xQPO,59jOOQO7+$V7+$VO$WQPO7+$VO$cQQO'#DPO$kQQO'#ChO$yQPO1G.qOOQO-E6q-E6qOOQO,59`,59`OOQO-E6r-E6rOOQO<<Gq<<GqO!ZQPO'#CrO%OQQO,59kO%WQQO,59XOOQO7+$]7+$]OOQO,59^,59^OOQO-E6p-E6p",
  stateData: "%c~OkOSlOSPOS~OTRO~OnTO~OWWOoZO~Om[O~OWWO~OZaOqbO~OcdOm[O~OWWOmeO~OZjO~O`kOulOb[Xc[Xm[Xv[X~ObnOcrXmrXvrX~OvpOcXimXi~OulObaacaamaavaa~ObnOcramravra~OvxOcXqmXq~OtyO^sX~O`kOulO^[Xt[X~O^|O~OtyO^sa~OulO^aataa~O",
  goto: "$TtPPPuyP}!QP!_!cP!g!tP!WP!WPP!}#T#Z#a#kPPPPPPP#qP#z$QTSOQTPOQRUPQYUQ`ZZgajknyT^W_T]W_ShajQrkQvnR}yUfajnTskyQQORVQQ_WRc_QzrR!OzQmfSum{R{sQohRwo_XUZajknyQiaRqjRtk",
  nodeNames: "\u26A0 Comment Document ObjectTypeDefinition TypeKeyword type NamedTypeDeclaration FieldsDefinition { ValueDefinition ValueName : Type NamedType > GenericType < ArrayType | }",
  maxTerm: 38,
  skippedNodes: [0, 1],
  repeatNodeCount: 5,
  tokenData: "&i~RlX^!ypq!yst#nxy$Vyz$V|}$[!P!Q$c![!]$i!]!^$n!^!_$s!_!`$x!`!a$}!a!b%S!c!}%X!}#O%l#R#S%X#T#o%w#o#p&Y#p#q&_#q#r&d#y#z!y$f$g!y#BY#BZ!y$IS$I_!y$I|$JO!y$JT$JU!y$KV$KW!y&FU&FV!y~#OYk~X^!ypq!y#y#z!y$f$g!y#BY#BZ!y$IS$I_!y$I|$JO!y$JT$JU!y$KV$KW!y&FU&FV!y~#sSP~OY#nZ;'S#n;'S;=`$P<%lO#n~$SP;=`<%l#n~$[Ol~R$cOtQvP~$fP!P!Q#n~$nOZ~P$sOvP~$xO`~~$}Oo~~%SO^~~%XOq~R%`SnQmP!Q![%X!c!}%X#R#S%X#T#o%X~%oP#P#Q%r~%wOu~P%|SmP!Q![%w!c!}%w#R#S%w#T#o%w~&_OW~~&dOb~~&iOc~",
  tokenizers: [0, 1],
  topRules: { "Document": [0, 2] },
  specialized: [{ term: 29, get: (value) => spec_name[value] || -1 }],
  tokenPrec: 0
});

// src/linter.ts
import { linter as lint } from "@codemirror/lint";
import { getDiagnostics } from "@liveblocks/schema";
var linter = lint(
  (view) => {
    const code = view.state.doc.toString();
    const diagnostics = getDiagnostics(code).map((diagnostic) => {
      var _a;
      return {
        from: diagnostic.range[0].offset,
        to: diagnostic.range[1].offset,
        message: diagnostic.message,
        severity: diagnostic.severity,
        actions: (_a = diagnostic.suggestions) == null ? void 0 : _a.map((suggestion) => {
          if (suggestion.type === "replace") {
            return {
              name: "Replace",
              apply: (view2, from, to) => {
                view2.dispatch({
                  changes: { from, to, insert: suggestion.name }
                });
              }
            };
          } else if (suggestion.type === "add-object-type-def") {
            return {
              name: "Add definition",
              apply: (view2) => {
                const insert = `

type ${suggestion.name} {
  
}
`;
                const to = code.length;
                const from = code.trimEnd().length;
                view2.dispatch({
                  changes: {
                    from: to,
                    insert
                  },
                  selection: { anchor: from + insert.length - 3 }
                });
              }
            };
          } else {
            return;
          }
        }).filter(Boolean)
      };
    });
    return diagnostics;
  },
  { delay: 200 }
);

// src/index.ts
var parentheses = "( )";
var braces = "{ }";
var angleBrackets = "< >";
var punctuations = `${parentheses} ${braces} ${angleBrackets} : = ? |`;
var blocks = `FieldsDefinition ${braces}`;
var generic = `GenericType ${angleBrackets}`;
var LiveblocksSchema = LRLanguage.define({
  name: "liveblocks-schema",
  parser: parser.configure({
    props: [
      indentNodeProp.add({
        [blocks]: delimitedIndent({ closing: "}", align: true })
      }),
      indentNodeProp.add({
        [generic]: delimitedIndent({ closing: ">", align: false })
      }),
      foldNodeProp.add({
        [blocks]: foldInside
      }),
      styleTags({
        Comment: t.lineComment,
        ValueName: t.name,
        NamedType: t.typeName,
        GenericType: t.special(t.typeName),
        NamedTypeDeclaration: t.function(t.typeName),
        type: t.keyword,
        separator: t.separator,
        comma: t.separator,
        [parentheses]: t.paren,
        [braces]: t.brace,
        [angleBrackets]: t.angleBracket,
        [punctuations]: t.punctuation
      })
    ]
  }),
  languageData: {
    commentTokens: { line: "#" },
    closeBrackets: { brackets: ["{", "<", "[", "(", '"'] },
    indentOnInput: /^\s*(\{|\})$/
  }
});
function liveblocksSchema() {
  return new LanguageSupport(LiveblocksSchema);
}
export {
  LiveblocksSchema,
  linter,
  liveblocksSchema
};
